__version__ = "0.5.2"

"""
Version: 0.5.2: Add canvas init from image and setters from scalars
Version: 0.5.1: Add canvas.rgb getter/setter to quickly get non-alpha img
Version: 0.5.0: Resize function in canvas.resize (removed transform.scale)
Version: 0.4.9: Loading from file now respects an alpha channel
Version: 0.4.8: Fix load from file to have correct pixels_per_unit
Version: 0.4.7: Fix copy/blank error
Version: 0.4.6: polyLines can now be filled
Version: 0.4.5: Added glow filter, slow but pretty
Version: 0.4.4: Colors now take hex code. Fix numpy error w/new version.
Version: 0.4.3: canvas.show() now returns the keycode.
Version: 0.4.2: Add helper load function directly from pixelhouse.
Version: 0.4.1: Add helper functions to see all the color palettes.
Version: 0.4.0: transform_length was off by a factor of two. Ouch. Fixed all the examples and code (mostly circle radii). This may break older code.
Version: 0.3.3: 100% Coverage. Fixed tiny bugs while building unit tests
Version: 0.3.2: Removed unnecessary debug statement and added animated pixelhouse logo example.
Version: 0.3.1: Fixed the manifest so the font files load.
Version: 0.3.0: Cleaned up easing API, now much cleaner and _easier_ to use. Ha.
Version: 0.2.2: Canvas now operates as an array (you can index and change the values as a numpy array). canvas._img is now simply canvas.img
Version: 0.2.1: Add three new functions, hstack, vstack, and grid stack
Version: 0.2.0: Improved gradients (multi colors, discrete, and transparency). Changed gradient API. Fixed a few canvas bugs. Added canvas properties like C.xmax.
Version: 0.1.5: Escape animations with any key
Version: 0.1.4: Added a wave effect transform
Version: 0.1.3: Moved ColorLoversPalette to just palette
Version: 0.1.2: Fixed MANIFEST! Missing assets
Version: 0.1.1: Refactored .filter to .filters to prevent namespace collision
Version: 0.1.0: First release! Added subpixel resolution to prior work.
"""



from .artist import Artist
from .canvas import Canvas, load
from .animation import Animation

from . import transform
from . import motion
from . import filters
from . import color
from . import gradient

from .primitives import circle, line, ellipse, rectangle, polyline, text
from .color import palette, palette_rectangles, palette_blocks
from .io import canvas2gif, canvas2mp4
from .canvas import hstack, vstack, gridstack

from ._version import __version__



import os
import json
import numpy

_script_path = os.path.dirname(os.path.abspath(__file__))


class NamedColors:
    """
    Named colors from matplotlib
    """

    def __init__(self, f_colors=None):

        if f_colors is None:
            f_colors = os.path.join(
                _script_path, "named_colors", "matplotlib.json"
            )

        self.f_colors = f_colors
        self.colors = None

    def _load_colors(self):
        # Lazy load the colors

        with open(self.f_colors) as FIN:
            raw = FIN.read()

        self.colors = json.loads(raw)

    def __call__(self, name):
        if self.colors is None:
            self._load_colors()

        if name in self.colors:
            color = self.colors[name]

        elif name and name[0] == "#":
            name = name.upper()
            r = int(name[1:3], 16)
            g = int(name[3:5], 16)
            b = int(name[5:7], 16)
            color = [r, g, b]

        else:
            color_list = self.colors.keys()
            msg = f"{color_list} are known, {name} is unknown and not hex."
            raise KeyError(msg)

        # Add in the alpha channel
        return color

    def __len__(self):
        return len(self.colors)



import setuptools
import os

__local__ = os.path.abspath(os.path.dirname(__file__))

f_version = os.path.join(__local__, "pixelhouse", "_version.py")
exec(open(f_version).read())

# Get the long description from the relevant file
long_description = """pixelhouse
=================================
A minimalist drawing library for making beautiful animations in python
"""

setuptools.setup(
    name="pixelhouse",
    packages=setuptools.find_packages(),
    # Include package data...
    include_package_data=True,
    description="A minimalist drawing library for making beautiful animations in python",
    long_description=long_description,
    version=__version__,
    # The project's main homepage.
    url="https://github.com/thoppe/pixelhouse",
    # download_url='https://github.com/...',
    # Author details
    author="Travis Hoppe",
    author_email="travis.hoppe+pixelhouse@gmail.com",
    # Choose your license
    license="CC-SA",
    install_requires=["opencv-python", "Pillow", "imageio", "numpy", "scipy", "tqdm"],
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5  -Production/Stable
        "Development Status :: 5 - Production/Stable",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Healthcare Industry",
        "Intended Audience :: Education",
        "Intended Audience :: Financial and Insurance Industry",
        "Natural Language :: English",
        #'Topic :: Text Processing',
        #'Topic :: Text Processing :: Filters',
        #'Topic :: Utilities',
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.6",
    ],
    # What does your project relate to?
    keywords="art",
    test_suite="nose.collector",
    tests_require=["nose"],
)



import os
import json
import numpy as np
import collections

_script_path = os.path.dirname(os.path.abspath(__file__))


class ColorLoversPalette:
    """
    Nice palettes inspired by https://www.colourlovers.com/
    """

    def __init__(self, colorset=1000):
        # Load the top 1000 by default

        f_palette = os.path.join(_script_path, f"palettes/{colorset}.json")
        self.f_palette = f_palette
        self.colors = None

    def _load_colors(self):
        # Lazy load the colors
        with open(self.f_palette) as FIN:
            raw = FIN.read()

        self.colors = json.loads(raw)

    def __call__(self, n):
        if self.colors is None:
            self._load_colors()

        if n > len(self.colors):
            msg = f"Only {len(self)} palettes known, requested number {n}"
            raise KeyError(msg)

        pal = [x + [255] for x in self.colors[n]]

        return pal

    def __len__(self):
        return len(self.colors)


def palette_rectangles(n, show_number=True, width=400, ratio=7):
    """
    Returns a canvas that shows the what palette "n" looks like.

    Example: 
         palette_rectangle(42).show()
    """

    from .. import Canvas, rectangle, text, palette

    extent = 4
    C = Canvas(width, width // ratio, extent=extent, bg="w")

    pal = palette(n)
    segments = np.linspace(-extent, extent, len(pal) + 2)

    for x0, x1, c in zip(segments, segments[1:], pal):
        C += rectangle(x0, -extent / ratio, x1, extent / ratio, color=c)

    if show_number:
        C += text(x=0.83 * extent, text=str(n), color="k", font_size=0.75)

    return C


def palette_blocks(palette_numbers, columns=2, show_number=True, width=400):
    """
    Returns a canvas that shows what a list (palette_numbers) of palettes 
    look like.
    
    Example: 
         palette_blocks(range(20)).show()
    """

    from .. import gridstack

    if not palette_numbers:
        raise ValueError("Need to pass in a list of palette numbers to show")

    if len(palette_numbers) < columns:
        raise ValueError("Need to pass in at least as many palettes as columns")

    blocks = collections.defaultdict(list)
    row = 0

    for n in palette_numbers:
        canvas = palette_rectangles(n, show_number=show_number, width=width)
        blocks[row].append(canvas)
        row = (row + 1) % ((len(palette_numbers) + 1) // columns)

    blocks = [blocks[k] for k in blocks]

    # Pad out irregular number of items
    m = len(blocks[0])

    while len(blocks[-1]) != m:
        blocks[-1].append(canvas.blank())

    C = gridstack(blocks)
    return C
